@@ -14,18 +14,21 @@
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
 
-from gi.repository import Gtk, Gdk, GdkPixbuf, GObject
-import threading
-import os
-import time
 import logging
+import os
+import threading
+import time
+
+from gi.repository import Gdk, GdkPixbuf, GObject, Gtk
+
+from variety.profile import get_profile_wm_class
+from variety.Util import Util, on_gtk
+
+logger = logging.getLogger("variety")
 
-logger = logging.getLogger('variety')
 
 class ThumbsWindow(Gtk.Window):
-    __gsignals__ = {
-        'clicked': (GObject.SIGNAL_RUN_FIRST, None, (str, Gtk.Widget, object))
-    }
+    __gsignals__ = {"clicked": (GObject.SIGNAL_RUN_FIRST, None, (str, Gtk.Widget, object))}
 
     LEFT = 1
     RIGHT = 2
@@ -36,6 +39,8 @@ class ThumbsWindow(Gtk.Window):
         logger.debug(lambda: "Creating thumb window %s, %d" % (str(self), time.time()))
         super(ThumbsWindow, self).__init__()
 
+        self.set_wmclass(get_profile_wm_class(), get_profile_wm_class())
+
         self.running = True
 
         self.set_decorated(False)
@@ -46,7 +51,9 @@ class ThumbsWindow(Gtk.Window):
         self.screen_height = self.screen.get_height()
 
         self.position = position
-        self.breadth = int(breadth * (1 if self.is_horizontal() else float(self.screen_width) / self.screen_height))
+        self.breadth = int(
+            breadth * (1 if self.is_horizontal() else float(self.screen_width) / self.screen_height)
+        )
 
         self.box = Gtk.HBox(False, 0) if self.is_horizontal() else Gtk.VBox(False, 0)
 
@@ -84,12 +91,14 @@ class ThumbsWindow(Gtk.Window):
         eventbox = Gtk.EventBox()
         eventbox.set_visible(True)
         eventbox.add(self.scroll)
-        eventbox.set_events(Gdk.EventMask.ENTER_NOTIFY_MASK |
-                            Gdk.EventMask.LEAVE_NOTIFY_MASK |
-                            Gdk.EventMask.POINTER_MOTION_MASK)
-        eventbox.connect('enter-notify-event', mouse_enter)
-        eventbox.connect('leave-notify-event', mouse_leave)
-        eventbox.connect('motion-notify-event', mouse_motion)
+        eventbox.set_events(
+            Gdk.EventMask.ENTER_NOTIFY_MASK
+            | Gdk.EventMask.LEAVE_NOTIFY_MASK
+            | Gdk.EventMask.POINTER_MOTION_MASK
+        )
+        eventbox.connect("enter-notify-event", mouse_enter)
+        eventbox.connect("leave-notify-event", mouse_leave)
+        eventbox.connect("motion-notify-event", mouse_motion)
 
         self.add(eventbox)
 
@@ -116,9 +125,12 @@ class ThumbsWindow(Gtk.Window):
     def pin(self, widget=None):
         self.pinned = True
 
+    @on_gtk
     def start(self, images):
         self.images = images
 
+        self._show()
+
         thumbs_thread = threading.Thread(target=self._thumbs_thread)
         thumbs_thread.daemon = True
         thumbs_thread.start()
@@ -127,72 +139,62 @@ class ThumbsWindow(Gtk.Window):
         autoscroll_thread.daemon = True
         autoscroll_thread.start()
 
+    def _show(self):
+        self.set_default_size(1, 1)
+        logger.debug(lambda: "Showing thumb window %s, %d" % (str(self), time.time()))
+        if self.position == ThumbsWindow.BOTTOM:
+            self.move(self.screen_width // 2, self.screen_height - self.breadth)
+        elif self.position == ThumbsWindow.TOP:
+            self.move(self.screen_width // 2, 0)
+        elif self.position == ThumbsWindow.LEFT:
+            self.move(0, self.screen_height // 2)
+        elif self.position == ThumbsWindow.RIGHT:
+            self.move(self.screen_width - self.screen_height, self.screen_height // 2)
+        else:
+            raise Exception("Unsupported thumbs position: " + str(self.position))
+        self.show_all()
+
     def _thumbs_thread(self):
         logger.debug(lambda: "Starting thumb thread %s, %d" % (str(self), time.time()))
         try:
             self.total_width = 0
-            shown = False
 
             for i, file in enumerate(self.images):
                 if not self.running:
-                    try:
-                        Gdk.threads_enter()
-                        self.destroy()
-                    finally:
-                        Gdk.threads_leave()
+                    Util.add_mainloop_task(self.destroy)
                     return
 
-                if not shown:
-                    try:
-                        Gdk.threads_enter()
-                        self.set_default_size(10, 10)
-                        logger.debug(lambda: "Showing thumb window %s, %d" % (str(self), time.time()))
-                        self.show_all()
-                        if self.position == ThumbsWindow.BOTTOM:
-                            self.move(self.screen_width // 2, self.screen_height - self.breadth)
-                        elif self.position == ThumbsWindow.TOP:
-                            self.move(self.screen_width // 2, 0)
-                        elif self.position == ThumbsWindow.LEFT:
-                            self.move(0, self.screen_height // 2)
-                        elif self.position == ThumbsWindow.RIGHT:
-                            self.move(self.screen_width - self.screen_height, self.screen_height // 2)
-                        else:
-                            raise Exception("Unsupported thumbs position: " + str(self.position))
-                        shown = True
-                    finally:
-                        Gdk.threads_leave()
-
-                self.add_image(file, gdk_thread=False, at_front=False)
-
-                # we must yield from time to time, or GTK/cairo errors abound
-                still_visible = self.total_width < (self.screen_width if self.is_horizontal() else self.screen_height)
-                time.sleep(0.02 if still_visible else 0.02)
-
+                self.add_image(file, at_front=False)
                 self.image_count = i
 
         except Exception:
             logger.exception(lambda: "Error while creating thumbs:")
 
-    def add_image(self, file, gdk_thread=False, at_front=False):
+    def add_image(self, file, at_front=False):
         try:
             if self.is_horizontal():
                 pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(file, 10000, self.breadth)
             else:
                 pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(file, self.breadth, 10000)
         except Exception:
-            logger.warning(lambda: "Could not create thumbnail for file %s. File may be missing or invalid." % file)
+            logger.warning(
+                lambda: "Could not create thumbnail for file %s. File may be missing or invalid."
+                % file
+            )
             pixbuf = None
 
-        try:
-            if not gdk_thread:
-                Gdk.threads_enter()
-
-            image_size = 0 if not pixbuf else pixbuf.get_width() if self.is_horizontal() else pixbuf.get_height()
+        def _go():
+            image_size = (
+                0
+                if not pixbuf
+                else pixbuf.get_width()
+                if self.is_horizontal()
+                else pixbuf.get_height()
+            )
 
             thumb = Gtk.Image()
             if pixbuf:
                 thumb.set_from_pixbuf(pixbuf)
-    #            thumb.set_from_icon_name("folder", Gtk.IconSize.MENU)
             thumb.set_visible(True)
 
             overlay = Gtk.Overlay()
@@ -201,16 +203,20 @@ class ThumbsWindow(Gtk.Window):
 
             eventbox = Gtk.EventBox()
             eventbox.set_visible(True)
+
             def click(widget, event, file=file):
                 self.emit("clicked", file, widget, event)
+
             eventbox.connect("button-release-event", click)
             eventbox.add(overlay)
 
-            image_info = {"file": file,
-                          "eventbox": eventbox,
-                          "thumb": thumb,
-                          "size": image_size,
-                          "overlay": overlay}
+            image_info = {
+                "file": file,
+                "eventbox": eventbox,
+                "thumb": thumb,
+                "size": image_size,
+                "overlay": overlay,
+            }
             if at_front:
                 image_info["start"] = 0
                 for info in self.all:
@@ -224,7 +230,11 @@ class ThumbsWindow(Gtk.Window):
 
             self.total_width += image_size
 
-            adj = self.scroll.get_hadjustment() if self.is_horizontal() else self.scroll.get_vadjustment()
+            adj = (
+                self.scroll.get_hadjustment()
+                if self.is_horizontal()
+                else self.scroll.get_vadjustment()
+            )
             scrollbar_at_start = not adj or adj.get_value() <= adj.get_lower() + 20
 
             self.box.pack_start(eventbox, False, False, 0)
@@ -232,7 +242,11 @@ class ThumbsWindow(Gtk.Window):
             if at_front:
                 self.box.reorder_child(eventbox, 0)
                 # get adj again - we just added at front, scrollbar might have appeared
-                adj = self.scroll.get_hadjustment() if self.is_horizontal() else self.scroll.get_vadjustment()
+                adj = (
+                    self.scroll.get_hadjustment()
+                    if self.is_horizontal()
+                    else self.scroll.get_vadjustment()
+                )
                 if adj:
                     if scrollbar_at_start:
                         adj.set_value(adj.get_lower())
@@ -241,17 +255,21 @@ class ThumbsWindow(Gtk.Window):
 
             self.update_size()
 
-        finally:
-            if not gdk_thread:
-                Gdk.threads_leave()
+            if file == self.active_file or position == self.active_position:
+                self.mark_active(self.active_file, self.active_position)
 
-        if file == self.active_file or position == self.active_position:
-            self.mark_active(self.active_file, self.active_position)
+        Util.add_mainloop_task(_go)
 
     def update_size(self):
-        if self.total_width < (self.screen_width if self.is_horizontal() else self.screen_height) + 1000:
+        if (
+            self.total_width
+            < (self.screen_width if self.is_horizontal() else self.screen_height) + 1000
+        ):
             if self.position == ThumbsWindow.BOTTOM:
-                self.move(max(0, (self.screen_width - self.total_width) // 2), self.screen_height - self.breadth)
+                self.move(
+                    max(0, (self.screen_width - self.total_width) // 2),
+                    self.screen_height - self.breadth,
+                )
                 self.scroll.set_min_content_width(min(self.total_width, self.screen_width))
             elif self.position == ThumbsWindow.TOP:
                 self.move(max(0, (self.screen_width - self.total_width) // 2), 0)
@@ -260,30 +278,26 @@ class ThumbsWindow(Gtk.Window):
                 self.move(0, max(0, (self.screen_height - self.total_width) // 2))
                 self.scroll.set_min_content_height(min(self.total_width, self.screen_height))
             elif self.position == ThumbsWindow.RIGHT:
-                self.move(self.screen_width - self.breadth, max(0, (self.screen_height - self.total_width) // 2))
+                self.move(
+                    self.screen_width - self.breadth,
+                    max(0, (self.screen_height - self.total_width) // 2),
+                )
                 self.scroll.set_min_content_height(min(self.total_width, self.screen_height))
 
     # TODO this method is buggy when width < screen and scrollbar not shown - a blank space remains
-    def remove_image(self, image, gdk_thread=True):
-        try:
-            if not gdk_thread:
-                Gdk.threads_enter()
+    @on_gtk
+    def remove_image(self, image):
+        for info in self.all:
+            if info["file"] == image:
+                eventbox = info["eventbox"]
+                thumb = info["thumb"]
+                self.box.remove(eventbox)
+                eventbox.destroy()
+                thumb.destroy()
+                self.total_width -= info["size"]
+                self.update_size()
 
-            for info in self.all:
-                if info["file"] == image:
-                    eventbox = info["eventbox"]
-                    thumb = info["thumb"]
-                    self.box.remove(eventbox)
-                    eventbox.destroy()
-                    thumb.destroy()
-                    self.total_width -= info["size"]
-                    self.update_size()
-
-            self.all = [info for info in self.all if info["file"] != image]
-
-        finally:
-            if not gdk_thread:
-                Gdk.threads_leave()
+        self.all = [info for info in self.all if info["file"] != image]
 
     def mark_active(self, file=None, position=None):
         def _mark():
@@ -326,11 +340,11 @@ class ThumbsWindow(Gtk.Window):
                         cr.rectangle(0, 0, image_size, 5)
                     else:
                         cr.rectangle(0, 0, 5, image_size)
-                    cr.set_source_rgba(255.0/255, 105.0/255, 44.0/255)
+                    cr.set_source_rgba(255.0 / 255, 105.0 / 255, 44.0 / 255)
                     cr.fill()
                     return False
 
-                self.mark.connect('draw', _draw_callback)
+                self.mark.connect("draw", _draw_callback)
                 self.mark.set_visible(True)
 
                 self.marked_info["overlay"].add_overlay(self.mark)
@@ -388,19 +402,21 @@ class ThumbsWindow(Gtk.Window):
             x = self.mouse_position[0]
             y = self.mouse_position[1]
 
-            try:
-                Gdk.threads_enter()
+            def _go():
                 if self.is_horizontal() and y > 0:
-                    self.autoscroll_step(self.scroll.get_hadjustment(), self.scroll.get_min_content_width(), x)
+                    self.autoscroll_step(
+                        self.scroll.get_hadjustment(), self.scroll.get_min_content_width(), x
+                    )
                 elif not self.is_horizontal() and x > 0:
-                    self.autoscroll_step(self.scroll.get_vadjustment(), self.scroll.get_min_content_height(), y)
-            except Exception:
-                pass
-            finally:
-                Gdk.threads_leave()
+                    self.autoscroll_step(
+                        self.scroll.get_vadjustment(), self.scroll.get_min_content_height(), y
+                    )
+
+            Util.add_mainloop_task(_go)
 
             last_update = time.time()
 
+
 if __name__ == "__main__":
     images = []
     dir = "/usr/share/backgrounds"
@@ -409,18 +425,12 @@ if __name__ == "__main__":
         if os.path.isfile(file) and file.endswith(".jpg"):
             images.append(file)
 
-    print images
+    print(images)
 
     win = ThumbsWindow()
     win.connect("delete-event", Gtk.main_quit)
 
-    print "starting"
+    print("starting")
     win.start(images)
-
-    GObject.threads_init()
-    Gdk.threads_init()
-    Gdk.threads_enter()
-    print "gtk main"
+    print("gtk main")
     Gtk.main()
-    Gdk.threads_leave()
-
